/*
	Key_State version 0.1
	Released January 31th, 2011.

	Defines several procs, verbs, and vars on the client for use in registering and ckecking
	player key presses and held keys.
	*/
/*
	Usage Notes:

	Key_State is a small library which allows you to get answers to two questions that are
	important in gaming:
		Has the player pressed a key?
		Is the Player holding down a key?

	DM uses an event based environment where code is tied directly to player commands, called
	"verbs". In games with different environments, especially those with "game loops", the
	default event based player command system is a major obstacle. This library provides a
	way to passively log player commands, so that the game can actively respond on its own
	terms and at its own time.

	To use the library, define a set of macros in your dmf (skin) file. You should have two
	macros for each key you wish to use with the library, one which calls "key_up [key]" and
	the other which calls "key_down [key]", where [key] is a number, such as NORTH. The
	library uses bit-flags to store the key data, so you can track up to 16 different keys at
	any given time. Because the library uses bit-flags, key numbers must be powers of two:
		1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768.

	Note that the library currently does not work well with diagonal keys, such as NORTHEAST.
	Instead, players can hold the NORTH and EAST keys at the same time in order to move in
	those directions.

	To check if a player is holding down a key, call key_state([key]). This proc will return
	a true value if the player is holding down /any/ of the key bit-flags encoded in [key].
	In other words, if the player is holding the key NORTH, and you call key_state(NORTHEAST),
	the return value will be NORTH. If the player is holding the keys SOUTH and WEST, and you
	call key_state(NORTHEAST), the return value will be 0.

	The key_press() proc works in the same way. The difference between key_state and key_press
	key_state will only hold a key until the player releases it, whereas key_press will store
	a pressed key until the game tells it to clear that information. This means that you
	should call clear_press(~[key]) when you're done checking if the player has pressed [key].
	If you checked all key values at once, then call clear_press() without any argument to
	clear all key presses.

	Finally, the reset_keys() proc clears all player key information. Use this before giving
	control to the player. This will clear out any random keys he might have pressed while
	waiting to play. Otherwise your player-character might make strange random movements or
	attacks when given control.

	See the Demo files for more information.

	*/

client{
	var{
		// Do not change these variables.
		key_state = 0
		key_press = 0
		old_keys  = 0
		}
	verb{
		key_up(which as num){
			// Use this verb when defining keyboard macros, such as: North 1, or Ctrl+C 64
			set name = "key_up"
			set hidden = TRUE
			key_state &= ~which
			old_keys &= ~which
			if(which < 16){
				var/opposite = turn(which, 180)
				if(opposite & old_keys){
					key_state |= opposite
					old_keys &= ~opposite
					}
				}
			}
		key_down(which as num){
			// Use this verb to define the same macros as above,
			//                   but use the UP flag, such as: North+UP 1, or Ctrl+C+UP 64
			set name = "key_down"
			set hidden = TRUE
			if(which < 16){
				var/opposite = turn(which, 180)
				if(opposite & key_state){
					key_state &= ~opposite
					old_keys |= opposite
					}
				}
			key_state |= which
			key_press |= which
			}
		}
	proc{
		reset_keys(){
			// Use this before giving a player control of anything.
			key_state = 0
			old_keys = 0
			key_press = 0
			}
		key_state(which){
			// Use this to check if a player is holding down the key 'which'.
			return key_state & which
			}
		key_press(which){
			// Use this to check if a player has pressed the key 'which'.
			return key_press & which
			}
		clear_press(save_buttons){
			// Use this to clear out all key presses, except those encoded in 'save_buttons'.
			if(save_buttons){
				key_press &= save_buttons
				}
			else{
				key_press = 0
				}
			}
		}
	}